@@ -75,9 +75,6 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* All drivers implementing backing store need this */
 #include "mibstore.h"
 
-/* All drivers using the mi banking wrapper need this */
-#include "mibank.h"
-
 /* All drivers using the mi colormap manipulation need this */
 #include "micmap.h"
 
@@ -109,10 +106,9 @@ CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include <X11/Xproto.h>
 #include "scrnintstr.h"
 #include "servermd.h"
-#ifdef XFreeXDGA
-#define _XF86DGA_SERVER_
-#include <X11/extensions/xf86dgastr.h>
-#endif
+
+#include <stdlib.h>
+#include <unistd.h>
 
 /* Mandatory functions */
 static const OptionInfoRec *	NEOAvailableOptions(int chipid, int busid);
@@ -337,7 +333,6 @@ static IsaChipsets NEOISAchipsets[] = {
 
 /* The options supported by the Neomagic Driver */
 typedef enum {
-    OPTION_NOLINEAR_MODE,
     OPTION_NOACCEL,
     OPTION_SW_CURSOR,
     OPTION_NO_MMIO,
@@ -385,7 +380,6 @@ static const OptionInfoRec NEO_2070_Options[] = {
 };
 
 static const OptionInfoRec NEOOptions[] = {
-    { OPTION_NOLINEAR_MODE,"NoLinear",  OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_NOACCEL,	"NoAccel",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_SW_CURSOR,	"SWcursor",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_NO_MMIO,	"noMMIO",	OPTV_BOOLEAN,	{0}, FALSE },
@@ -482,7 +476,7 @@ NEOFreeRec(ScrnInfoPtr pScrn)
 {
     if (pScrn->driverPrivate == NULL)
 	return;
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 }
 
@@ -558,7 +552,7 @@ NEOProbe(DriverPtr drv, int flags)
 		    foundScreen = TRUE;
 		}
 	    }
-	    xfree(usedChips);
+	    free(usedChips);
 	}
     }
 
@@ -591,11 +585,11 @@ NEOProbe(DriverPtr drv, int flags)
 	    foundScreen = TRUE;
 	}
       }
-      xfree(usedChips);
+      free(usedChips);
     }
 #endif
 
-    xfree(devSections);
+    free(devSections);
     return foundScreen;
 }
 
@@ -666,6 +660,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     if (!vgaHWGetHWRec(pScrn))
 	return FALSE;
     hwp = VGAHWPTR(pScrn);
+    vgaHWSetStdFuncs(hwp);
 
     /* Allocate the NeoRec driverPrivate */
     if (!NEOGetRec(pScrn)) {
@@ -947,18 +942,17 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     xf86CollectOptions(pScrn, NULL);
     /* Process the options */
     if (nPtr->NeoChipset == NM2070) {
-	if (!(nPtr->Options = xalloc(sizeof(NEO_2070_Options))))
+	if (!(nPtr->Options = malloc(sizeof(NEO_2070_Options))))
 	    return FALSE;
 	memcpy(nPtr->Options, NEO_2070_Options, sizeof(NEO_2070_Options));
     } else {
-	if (!(nPtr->Options = xalloc(sizeof(NEOOptions))))
+	if (!(nPtr->Options = malloc(sizeof(NEOOptions))))
 	    return FALSE;
 	memcpy(nPtr->Options, NEOOptions, sizeof(NEOOptions));
     }
 
     xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, nPtr->Options);
 
-    xf86GetOptValBool(nPtr->Options, OPTION_NOLINEAR_MODE,&nPtr->noLinear);
     xf86GetOptValBool(nPtr->Options, OPTION_SW_CURSOR,&nPtr->swCursor);
     xf86GetOptValBool(nPtr->Options, OPTION_NO_MMIO,&nPtr->noMMIO);
     xf86GetOptValBool(nPtr->Options, OPTION_INTERN_DISP,&nPtr->internDisp);
@@ -1084,10 +1078,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     if (nPtr->lcdCenter)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG,
 		   "Video modes are centered on the display\n");
-    if (nPtr->noLinear)
-	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG, "using nonlinear mode\n");
-    else
-	xf86DrvMsg(pScrn->scrnIndex,X_DEFAULT, "using linear mode\n");
     if (nPtr->swCursor)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG, "using sofware cursor\n");
     if (nPtr->noMMIO)
@@ -1100,20 +1090,24 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
     if (nPtr->showcache)
 	xf86DrvMsg(pScrn->scrnIndex,X_CONFIG,
 		   "Show chache for debugging\n");
+
+    if (!xf86LoadSubModule(pScrn, "xaa")) {
+	xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Falling back to shadow\n");
+	nPtr->shadowFB = 1;
+    }
+
     if (nPtr->shadowFB) {
-	if (nPtr->noLinear) {
-    	    xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
-		       "Option \"ShadowFB\" ignored. Not supported without"
-		       " linear addressing\n");
-	    nPtr->shadowFB = FALSE;
-	    nPtr->rotate = 0;
-	} else {
-	    nPtr->noAccel = TRUE;
-	    xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
-		    "Using \"Shadow Framebuffer\" - acceleration disabled\n");
+	if (!xf86LoadSubModule(pScrn, "shadow")) {
+	    RETURN;
 	}
     }
 
+    if (nPtr->shadowFB) {
+        nPtr->noAccel = TRUE;
+        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, 
+                "Using \"Shadow Framebuffer\" - acceleration disabled\n");
+    }
+
     nPtr->NeoFbMapSize = linearSize * 1024;
     nPtr->NeoCursorOffset = CursorOff;
 
@@ -1205,7 +1199,7 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	linearRes[0].rBegin = nPtr->NeoLinearAddr;
 	linearRes[1].rEnd = nPtr->NeoLinearAddr + nPtr->NeoFbMapSize - 1;
 	if (xf86RegisterResources(nPtr->pEnt->index,linearRes,ResNone)) {
-	    nPtr->noLinear = TRUE; /* XXX */
+	    RETURN;
 	}
     }
 #endif
@@ -1333,17 +1327,6 @@ NEOPreInit(ScrnInfoPtr pScrn, int flags)
 	RETURN;
     }
 
-    if (!nPtr->noLinear) {
-	if (!xf86LoadSubModule(pScrn, "xaa")) 
-	    RETURN;
-    }
-
-    if (nPtr->shadowFB) {
-	if (!xf86LoadSubModule(pScrn, "shadow")) {
-	    RETURN;
-	}
-    }
-    
     if (!nPtr->swCursor) {
 	if (!xf86LoadSubModule(pScrn, "ramdac"))
 	    RETURN;
@@ -1476,22 +1459,11 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     miClearVisualTypes();
     
     /* Setup the visuals we support. */
-#if 0
     if (!miSetVisualTypes(pScrn->depth,
       		      miGetDefaultVisualMask(pScrn->depth),
 		      pScrn->rgbBits, pScrn->defaultVisual))
          return FALSE;
-#else
-    if (!miSetVisualTypes(pScrn->depth,
-      		      miGetDefaultVisualMask(pScrn->depth),
-		      pScrn->rgbBits, pScrn->defaultVisual))
-         return FALSE;
-    if (pScrn->depth > 8) {
-	if (!miSetVisualTypes(8, miGetDefaultVisualMask(8), 6,
-			      pScrn->defaultVisual))
-	    return FALSE;
-    }
-#endif
+
     if (!miSetPixmapDepths ()) return FALSE;
 
     /*
@@ -1509,7 +1481,7 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     
     if(nPtr->shadowFB) {
 	nPtr->ShadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
-	nPtr->ShadowPtr = xalloc(nPtr->ShadowPitch * height);
+	nPtr->ShadowPtr = malloc(nPtr->ShadowPitch * height);
 	displayWidth = nPtr->ShadowPitch / (pScrn->bitsPerPixel >> 3);
 	FBStart = nPtr->ShadowPtr;
     } else {
@@ -1527,8 +1499,7 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
         /* Fixup RGB ordering */
         visual = pScreen->visuals + pScreen->numVisuals;
         while (--visual >= pScreen->visuals) {
-	    if ((visual->class | DynamicClass) == DirectColor
-		&& visual->nplanes > 8) {
+	    if ((visual->class | DynamicClass) == DirectColor) {
 		visual->offsetRed = pScrn->offset.red;
 		visual->offsetGreen = pScrn->offset.green;
 		visual->offsetBlue = pScrn->offset.blue;
@@ -1552,134 +1523,105 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     nAcl->UseHWCursor = FALSE;
     nAcl->CursorAddress = -1;
     
-    if (nPtr->noLinear) {
-	miBankInfoPtr pBankInfo;
+    nAcl->cacheStart = -1;
+    nAcl->cacheEnd = -1;
+    xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+               "Using linear framebuffer at: 0x%08lX\n",
+               nPtr->NeoLinearAddr);
+    /* Setup pointers to free space in video ram */
+    allocatebase = (pScrn->videoRam << 10);
+    freespace = allocatebase - pScrn->displayWidth *
+        pScrn->virtualY * (pScrn->bitsPerPixel >> 3);
+    currentaddr = allocatebase;
+    xf86DrvMsg(scrnIndex, X_PROBED,
+               "%d bytes off-screen memory available\n", freespace);
 
-	/* Setup the vga banking variables */
-	pBankInfo = (miBankInfoPtr)xnfcalloc(sizeof(miBankInfoRec),1);
-	if (pBankInfo == NULL)
-	    return FALSE;
-	
-	pBankInfo->pBankA = hwp->Base;
-	pBankInfo->pBankB = (unsigned char *)hwp->Base;
-	pBankInfo->BankSize = 0x10000;
-	pBankInfo->nBankDepth = pScrn->depth;
-	
-	pBankInfo->SetSourceBank = (miBankProcPtr)NEOSetRead;
-	pBankInfo->SetDestinationBank =
-	    (miBankProcPtr)NEOSetWrite;
-	pBankInfo->SetSourceAndDestinationBanks =
-	    (miBankProcPtr)NEOSetReadWrite;
-	if (!miInitializeBanking(pScreen, pScrn->virtualX, pScrn->virtualY,
-				 pScrn->displayWidth, pBankInfo)) {
-	    xfree(pBankInfo);
-	    pBankInfo = NULL;
-	    return FALSE;
-	}
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO, "Using nonlinear mode\n");
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO, "Using software cursor in "
-		   "nonlinear mode\n");
+    if (nPtr->swCursor || !nPtr->NeoMMIOBase) {
+        xf86DrvMsg(scrnIndex, X_CONFIG,
+                   "Using Software Cursor.\n");
+    } else if (nPtr->NeoCursorMem <= freespace) {
+        currentaddr -= nPtr->NeoCursorMem;
+        freespace  -= nPtr->NeoCursorMem;
+        /* alignment */
+        freespace  -= currentaddr & 0x3FF;
+        currentaddr &= 0xfffffc00;
+        nAcl->CursorAddress = currentaddr;
+        xf86DrvMsg(scrnIndex, X_INFO,
+                   "Using H/W Cursor.\n"); 
     } else {
-	nAcl->cacheStart = -1;
-	nAcl->cacheEnd = -1;
-	xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		   "Using linear framebuffer at: 0x%08lX\n",
-		   nPtr->NeoLinearAddr);
-	/* Setup pointers to free space in video ram */
-	allocatebase = (pScrn->videoRam << 10);
-	freespace = allocatebase - pScrn->displayWidth *
-	    pScrn->virtualY * (pScrn->bitsPerPixel >> 3);
-	currentaddr = allocatebase;
-	xf86DrvMsg(scrnIndex, X_PROBED,
-		   "%d bytes off-screen memory available\n", freespace);
-
-	if (nPtr->swCursor || !nPtr->NeoMMIOBase) {
-	    xf86DrvMsg(scrnIndex, X_CONFIG,
-		       "Using Software Cursor.\n");
-	} else if (nPtr->NeoCursorMem <= freespace) {
-	    currentaddr -= nPtr->NeoCursorMem;
-	    freespace  -= nPtr->NeoCursorMem;
-	    /* alignment */
-	    freespace  -= currentaddr & 0x3FF;
-	    currentaddr &= 0xfffffc00;
-	    nAcl->CursorAddress = currentaddr;
-	    xf86DrvMsg(scrnIndex, X_INFO,
-		       "Using H/W Cursor.\n"); 
-	} else {
-	    xf86DrvMsg(scrnIndex, X_ERROR,
-		       "Too little space for H/W cursor.\n");
-	}
-	
-	if (!nPtr->noAccel && !nPtr->NeoMMIOBase)
-	  xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		     "Acceleration disabled when not using MMIO\n");
-
-	if (nPtr->overlay > 0){
-	    if (nPtr->overlay > freespace){
-		xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-			   "Can not reserve %d bytes for overlay. "
-			   "Resize to %d bytes.\n",
-			   nPtr->overlay, freespace);
-		nPtr->overlay = freespace;
-	    }
-	    currentaddr -= nPtr->overlay;
-	    freespace -= nPtr->overlay;
-	    nPtr->overlay_offset = currentaddr;
-	    xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Overlay at 0x%x\n",
-		       nPtr->overlay_offset);
-	}
-
-	nAcl->cacheStart = currentaddr - freespace;
-	nAcl->cacheEnd = currentaddr;
-	freespace = 0;
-	if (nAcl->cacheStart < nAcl->cacheEnd) {
-	    BoxRec AvailFBArea;
-	    int lines = nAcl->cacheEnd /
-		(pScrn->displayWidth * (pScrn->bitsPerPixel >> 3));
-	    if (!nPtr->noAccel && nPtr->NeoMMIOBase && lines > 1024) 
-		lines = 1024;
-	    AvailFBArea.x1 = 0;
-	    AvailFBArea.y1 = 0;
-	    AvailFBArea.x2 = pScrn->displayWidth;
-	    AvailFBArea.y2 = lines;
-	    xf86InitFBManager(pScreen, &AvailFBArea); 
-	    
-	    xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
-		       "Using %i scanlines of offscreen memory \n",
-		       lines - pScrn->virtualY);
-	}
-
-	/* Setup the acceleration primitives */
-	if (!nPtr->noAccel && nPtr->NeoMMIOBase) {
-	    Bool ret = FALSE;
-	    if (nAcl->cacheStart >= nAcl->cacheEnd) {
-		xf86DrvMsg(scrnIndex, X_ERROR,
-			   "Too little space for pixmap cache.\n");
-	    } 	    
-	    switch(nPtr->NeoChipset) {
-	    case NM2070 :
-		ret = Neo2070AccelInit(pScreen);
-		break;
-	    case NM2090 :
-	    case NM2093 :
-		ret = Neo2090AccelInit(pScreen);
-		break;
-	    case NM2097 :
-	    case NM2160 :
-		ret = Neo2097AccelInit(pScreen);
-		break;
-	    case NM2200 :
-	    case NM2230 :
-	    case NM2360 :
-	    case NM2380 :
-	        ret = Neo2200AccelInit(pScreen);
-		break;
-	    }
-	    xf86DrvMsg(pScrn->scrnIndex,X_INFO,
-		       "Acceleration %s Initialized\n",ret ? "" : "not");
-	} 
-
+        xf86DrvMsg(scrnIndex, X_ERROR,
+                   "Too little space for H/W cursor.\n");
     }
+    
+    if (!nPtr->noAccel && !nPtr->NeoMMIOBase)
+      xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                 "Acceleration disabled when not using MMIO\n");
+
+    if (nPtr->overlay > 0){
+        if (nPtr->overlay > freespace){
+            xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                       "Can not reserve %d bytes for overlay. "
+                       "Resize to %d bytes.\n",
+                       nPtr->overlay, freespace);
+            nPtr->overlay = freespace;
+        }
+        currentaddr -= nPtr->overlay;
+        freespace -= nPtr->overlay;
+        nPtr->overlay_offset = currentaddr;
+        xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Overlay at 0x%x\n",
+                   nPtr->overlay_offset);
+    }
+
+    nAcl->cacheStart = currentaddr - freespace;
+    nAcl->cacheEnd = currentaddr;
+    freespace = 0;
+    if (nAcl->cacheStart < nAcl->cacheEnd) {
+        BoxRec AvailFBArea;
+        int lines = nAcl->cacheEnd /
+            (pScrn->displayWidth * (pScrn->bitsPerPixel >> 3));
+        if (!nPtr->noAccel && nPtr->NeoMMIOBase && lines > 1024) 
+            lines = 1024;
+        AvailFBArea.x1 = 0;
+        AvailFBArea.y1 = 0;
+        AvailFBArea.x2 = pScrn->displayWidth;
+        AvailFBArea.y2 = lines;
+        xf86InitFBManager(pScreen, &AvailFBArea); 
+        
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, 
+                   "Using %i scanlines of offscreen memory \n",
+                   lines - pScrn->virtualY);
+    }
+
+    /* Setup the acceleration primitives */
+    if (!nPtr->noAccel && nPtr->NeoMMIOBase) {
+        Bool ret = FALSE;
+        if (nAcl->cacheStart >= nAcl->cacheEnd) {
+            xf86DrvMsg(scrnIndex, X_ERROR,
+                       "Too little space for pixmap cache.\n");
+        } 	    
+        switch(nPtr->NeoChipset) {
+        case NM2070 :
+            ret = Neo2070AccelInit(pScreen);
+            break;
+        case NM2090 :
+        case NM2093 :
+            ret = Neo2090AccelInit(pScreen);
+            break;
+        case NM2097 :
+        case NM2160 :
+            ret = Neo2097AccelInit(pScreen);
+            break;
+        case NM2200 :
+        case NM2230 :
+        case NM2360 :
+        case NM2380 :
+            ret = Neo2200AccelInit(pScreen);
+            break;
+        }
+        xf86DrvMsg(pScrn->scrnIndex,X_INFO,
+                   "Acceleration %s Initialized\n",ret ? "" : "not");
+    } 
+
     miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
     xf86SetSilkenMouse(pScreen);
@@ -1747,10 +1689,8 @@ NEOScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	xf86DPMSInit(pScreen, (DPMSSetProcPtr)NeoDisplayPowerManagementSet,
 		     0);
 
-    if (!nPtr->noLinear) {
-	pScrn->memPhysBase = (unsigned long)nPtr->NeoLinearAddr;
-	pScrn->fbOffset = 0;
-    }
+    pScrn->memPhysBase = (unsigned long)nPtr->NeoLinearAddr;
+    pScrn->fbOffset = 0;
     
     /* Wrap the current CloseScreen function */
     nPtr->CloseScreen = pScreen->CloseScreen;
@@ -1854,7 +1794,7 @@ NEOCloseScreen(int scrnIndex, ScreenPtr pScreen)
     if (nPtr->CursorInfo)
 	xf86DestroyCursorInfoRec(nPtr->CursorInfo);
     if (nPtr->ShadowPtr)
-	xfree(nPtr->ShadowPtr);
+	free(nPtr->ShadowPtr);
 
     pScrn->vtSema = FALSE;
     pScreen->CloseScreen = nPtr->CloseScreen;
@@ -1977,85 +1917,80 @@ neoMapMem(ScrnInfoPtr pScrn)
     NEOPtr nPtr = NEOPTR(pScrn);
     vgaHWPtr hwp = VGAHWPTR(pScrn);
 
-    if (!nPtr->noLinear) {
-	if (!nPtr->noMMIO) {
-	    if (nPtr->pEnt->location.type == BUS_PCI){
+    if (!nPtr->noMMIO) {
+        if (nPtr->pEnt->location.type == BUS_PCI){
 
 #ifndef XSERVER_LIBPCIACCESS
-		nPtr->NeoMMIOBase =
-		    xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
-				  nPtr->PciTag, nPtr->NeoMMIOAddr,
-				  0x200000L);
-		if (nPtr->NeoMMIOAddr2 != 0){
-		    nPtr->NeoMMIOBase2 =
-		        xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
-				      nPtr->PciTag, nPtr->NeoMMIOAddr2,
-				      0x100000L);
-		}
+            nPtr->NeoMMIOBase =
+                xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
+                              nPtr->PciTag, nPtr->NeoMMIOAddr,
+                              0x200000L);
+            if (nPtr->NeoMMIOAddr2 != 0){
+                nPtr->NeoMMIOBase2 =
+                    xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
+                                  nPtr->PciTag, nPtr->NeoMMIOAddr2,
+                                  0x100000L);
+            }
 
 #else
-		void** result = (void**)&nPtr->NeoMMIOBase;
-		int err = pci_device_map_range(nPtr->PciInfo,
-					       nPtr->NeoMMIOAddr,
-					       0x200000L,
-					       PCI_DEV_MAP_FLAG_WRITABLE,
-					       result);
-		if (err)
-		    return FALSE;
-		
-		if (nPtr->NeoMMIOAddr2 != 0){
-		    result = (void**)&nPtr->NeoMMIOBase2;
-		    err = pci_device_map_range(nPtr->PciInfo,
-						   nPtr->NeoMMIOAddr2,
-						   0x100000L,
-						   PCI_DEV_MAP_FLAG_WRITABLE,
-						   result);
+            void** result = (void**)&nPtr->NeoMMIOBase;
+            int err = pci_device_map_range(nPtr->PciInfo,
+                                           nPtr->NeoMMIOAddr,
+                                           0x200000L,
+                                           PCI_DEV_MAP_FLAG_WRITABLE,
+                                           result);
+            if (err)
+                return FALSE;
+            
+            if (nPtr->NeoMMIOAddr2 != 0){
+                result = (void**)&nPtr->NeoMMIOBase2;
+                err = pci_device_map_range(nPtr->PciInfo,
+                                               nPtr->NeoMMIOAddr2,
+                                               0x100000L,
+                                               PCI_DEV_MAP_FLAG_WRITABLE,
+                                               result);
 
-		    if (err) 
-			return FALSE;
-		}
+                if (err) 
+                    return FALSE;
+            }
 #endif
-	    } else
-		nPtr->NeoMMIOBase =
-		    xf86MapVidMem(pScrn->scrnIndex,
-				  VIDMEM_MMIO, nPtr->NeoMMIOAddr,
-				  0x200000L);
-	    if (nPtr->NeoMMIOBase == NULL)
-	        return FALSE;
-	}
-
-	if (nPtr->pEnt->location.type == BUS_PCI)
-
-#ifndef XSERVER_LIBPCIACCESS
-	    nPtr->NeoFbBase =
-		xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-			      nPtr->PciTag,
-			      (unsigned long)nPtr->NeoLinearAddr,
-			      nPtr->NeoFbMapSize);
-#else
-	{
-	    void** result = (void**)&nPtr->NeoFbBase;
-	    int err = pci_device_map_range(nPtr->PciInfo,
-					   nPtr->NeoLinearAddr,
-					   nPtr->NeoFbMapSize,
-					   PCI_DEV_MAP_FLAG_WRITABLE |
-					   PCI_DEV_MAP_FLAG_WRITE_COMBINE,
-					   result);
-	    if (err)
-		return FALSE;
-	}
-#endif
-	else
-	    nPtr->NeoFbBase =
-		xf86MapVidMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-			      (unsigned long)nPtr->NeoLinearAddr,
-			      nPtr->NeoFbMapSize);
-	if (nPtr->NeoFbBase == NULL)
-	    return FALSE;
-    } else {
-	/* In paged mode Base is the VGA window at 0xA0000 */
-	nPtr->NeoFbBase = hwp->Base;
+        } else
+            nPtr->NeoMMIOBase =
+                xf86MapVidMem(pScrn->scrnIndex,
+                              VIDMEM_MMIO, nPtr->NeoMMIOAddr,
+                              0x200000L);
+        if (nPtr->NeoMMIOBase == NULL)
+            return FALSE;
     }
+
+    if (nPtr->pEnt->location.type == BUS_PCI)
+
+#ifndef XSERVER_LIBPCIACCESS
+        nPtr->NeoFbBase =
+            xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+                          nPtr->PciTag,
+                          (unsigned long)nPtr->NeoLinearAddr,
+                          nPtr->NeoFbMapSize);
+#else
+    {
+        void** result = (void**)&nPtr->NeoFbBase;
+        int err = pci_device_map_range(nPtr->PciInfo,
+                                       nPtr->NeoLinearAddr,
+                                       nPtr->NeoFbMapSize,
+                                       PCI_DEV_MAP_FLAG_WRITABLE |
+                                       PCI_DEV_MAP_FLAG_WRITE_COMBINE,
+                                       result);
+        if (err)
+            return FALSE;
+    }
+#endif
+    else
+        nPtr->NeoFbBase =
+            xf86MapVidMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+                          (unsigned long)nPtr->NeoLinearAddr,
+                          nPtr->NeoFbMapSize);
+    if (nPtr->NeoFbBase == NULL)
+        return FALSE;
     return TRUE;
 }
 
@@ -2068,32 +2003,30 @@ neoUnmapMem(ScrnInfoPtr pScrn)
 {
     NEOPtr nPtr = NEOPTR(pScrn);
 
-    if (!nPtr->noLinear) {
 #ifndef XSERVER_LIBPCIACCESS
-      if (nPtr->NeoMMIOBase)
-	  xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase,
-			  0x200000L);
+    if (nPtr->NeoMMIOBase)
+        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase,
+                        0x200000L);
 #else
-      if (nPtr->NeoMMIOBase)
-	  pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase, 0x200000L);
+    if (nPtr->NeoMMIOBase)
+        pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase, 0x200000L);
 #endif
-      nPtr->NeoMMIOBase = NULL;
+    nPtr->NeoMMIOBase = NULL;
 #ifndef XSERVER_LIBPCIACCESS
-      if (nPtr->NeoMMIOBase2)
-	  xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase2,
-			  0x100000L);
+    if (nPtr->NeoMMIOBase2)
+        xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoMMIOBase2,
+                        0x100000L);
 #else
-      if (nPtr->NeoMMIOBase2)
-	  pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase2, 0x100000L);
+    if (nPtr->NeoMMIOBase2)
+        pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoMMIOBase2, 0x100000L);
 #endif
-      nPtr->NeoMMIOBase2 = NULL;
+    nPtr->NeoMMIOBase2 = NULL;
 #ifndef XSERVER_LIBPCIACCESS
-      xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoFbBase,
-		    nPtr->NeoFbMapSize); 
+    xf86UnMapVidMem(pScrn->scrnIndex, (pointer)nPtr->NeoFbBase,
+                    nPtr->NeoFbMapSize); 
 #else
-      pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoFbBase, nPtr->NeoFbMapSize);
+    pci_device_unmap_range(nPtr->PciInfo, (pointer)nPtr->NeoFbBase, nPtr->NeoFbMapSize);
 #endif
-    }
     nPtr->NeoFbBase = NULL;
     
     return TRUE;
@@ -2489,7 +2422,7 @@ neoRestore(ScrnInfoPtr pScrn, vgaRegPtr VgaReg, NeoRegPtr restore,
      * In some rare cases a lockup might occur if we don't delay
      * here. (Reported by Miles Lane)
      */
-    xf86UDelay(200000);
+    usleep(200000);
     /*
      * Disable horizontal and vertical graphics and text expansions so
      * that vgaHWRestore works properly.
@@ -2502,7 +2435,7 @@ neoRestore(ScrnInfoPtr pScrn, vgaRegPtr VgaReg, NeoRegPtr restore,
      * Sleep for 200ms to make sure that the two operations above have
      * had time to take effect.
      */
-    xf86UDelay(200000);
+    usleep(200000);
     /*
      * This function handles restoring the generic VGA registers.  */
     vgaHWRestore(pScrn, VgaReg,
@@ -2665,7 +2598,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
      */
     NeoStd->Attribute[16] = 0x01;
 
-    switch (pScrn->depth) { /*@!@*/
+    switch (pScrn->depth) {
     case  8 :
 	NeoStd->CRTC[0x13] = pScrn->displayWidth >> 3;
 	NeoNew->ExtCRTOffset   = pScrn->displayWidth >> 11;
@@ -2707,8 +2640,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
     /* If they are used, enable linear addressing and/or enable MMIO. */
     NeoNew->SysIfaceCntl2 = 0x00;
-    if (!nPtr->noLinear)
-	NeoNew->SysIfaceCntl2 |= 0x80;
+    NeoNew->SysIfaceCntl2 |= 0x80;
     if (!nPtr->noMMIO)
 	NeoNew->SysIfaceCntl2 |= 0x40;
 
@@ -2933,7 +2865,7 @@ neoModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     if (NeoNew->reg) {
 	xf86DrvMsg(pScrn->scrnIndex, X_WARNING,
 		   "Non-NULL reg in NeoInit: reg=%p\n", (void *)NeoNew->reg);
-	xfree(NeoNew->reg);
+	free(NeoNew->reg);
 	NeoNew->reg = NULL;
     }
 
